export declare type FontProps = {
    fontSize?: number;
    fontFamily?: string;
    fontStyle?: 'normal' | 'italic';
    fontWeight?: 'normal' | 'bold' | 100 | 200 | 300 | 400 | 500 | 600 | 700 | 800 | 900;
};
export declare type TextStyleProps = FontProps & {
    color?: string;
    lineHeight?: number;
    textAlign?: 'auto' | 'left' | 'right' | 'center' | 'justify';
    textAlignVertical?: 'auto' | 'top' | 'bottom' | 'center';
    letterSpacing?: number;
    textTransform?: 'none' | 'uppercase' | 'lowercase' | 'capitalize';
};
export declare type ItemProps = {
    width?: number;
    height?: number;
    x?: number;
    y?: number;
    z?: number;
    implicitWidth?: number;
    implicitHeight?: number;
    opacity?: number;
    visible?: boolean;
    clip?: boolean;
    antialiasing?: boolean;
    baselineOffset?: number;
};
export declare type TransformProps = {
    scale?: number;
    rotation?: number;
};
export declare type LayoutProps = {
    alignment?: number | string;
    minimumWidth?: number;
    minimumHeight?: number;
    preferredWidth?: number;
    preferredHeight?: number;
    maximumWidth?: number;
    maximumHeight?: number;
    fillHeight?: number;
    fillWidth?: number;
    row?: number;
    rowSpan?: number;
    columnSpan?: number;
    margins?: number;
    leftMargin?: number;
    topMargin?: number;
    rightMargin?: number;
    bottomMargin?: number;
};
export declare type Style = TextStyleProps & ItemProps & TransformProps & LayoutProps & {
    [key: string]: any;
};
export declare function flattenStyle(style: any): any;
export declare function mapStyleToProps(style: Style): Qml.QmlProps;
export declare function setStyle(qmlElement: Qml.QmlElement, style: Style): void;
declare const _default: {
    flattenStyle: typeof flattenStyle;
    mapStyleToProps: typeof mapStyleToProps;
    setStyle: typeof setStyle;
};
export default _default;
